﻿using System;
using System.Windows;
using System.Threading;
using System.Windows.Threading;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.button.Click += button_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 테스트 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 테스트 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            IsEnabled = false;

            Thread thread = new Thread(new ThreadStart(ProcessTest));

            thread.Start();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 테스트 처리하기 - ProcessTest()

        /// <summary>
        /// 테스트 처리하기
        /// </summary>
        private void ProcessTest()
        {

            for(int i = 1; i <= 10; i++)
            {
                Thread.Sleep(100);
            
                DispatcherOperation dispatcherOperation = Dispatcher.BeginInvoke
                (
                    DispatcherPriority.Normal,
                    new Action<string, int>
                    (
                        (message, value) => {
                            this.textBox.Text = string.Format("{0} : {1}", message, value);
                        }
                    ),
                    "테스트",
                    i
                );

                DispatcherOperationStatus dispatcherOperationStatus = dispatcherOperation.Status;

                while(dispatcherOperationStatus != DispatcherOperationStatus.Completed)
                {
                    dispatcherOperationStatus = dispatcherOperation.Wait(TimeSpan.FromMilliseconds(1000));

                    if(dispatcherOperationStatus == DispatcherOperationStatus.Aborted)
                    {
                        // Abort시 처리한다.
                    }
                }
            }

            Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(() => { IsEnabled = true; }));
        }

        #endregion
    }
}